﻿using System.Windows;
using System.Windows.Controls;
using Microsoft.WindowsAPICodePack.ApplicationServices;

namespace PowerManagement
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();

            this.Loaded += (s, e) =>
                {
                    CheckPowerStatus();
                };

            // Änderung des Energieplans
            PowerManager.PowerPersonalityChanged += (s, e) =>
            {
                CheckPowerStatus();
            };

            // Änderung der Stromquelle
            PowerManager.PowerSourceChanged += (s, e) =>
            {
                CheckPowerStatus();
            };

            // Änderung des Batterieladezustands
            PowerManager.BatteryLifePercentChanged += (s, e) =>
            {
                CheckPowerStatus();
            };
        }

        private void CheckPowerStatus()
        {
            _statusMessages.Children.Clear();

            if (PowerManager.IsBatteryPresent)
            {
                AddMessage("Batterie ist vorhanden");
            }
            else
            {
                AddMessage("Keine Batterie ist vorhanden");
            }

            if (PowerManager.IsMonitorOn)
            {
                AddMessage("Monitor ist aktiv");
            }
            else
            {
                AddMessage("Monitor ist nicht aktiv");
            }

            AddMessage("Stromquelle: " + PowerManager.PowerSource.ToString());

            AddMessage("Batterieladezustanst: " + PowerManager.BatteryLifePercent.ToString() + " %");

            AddMessage("Energieplan: " + PowerManager.PowerPersonality.ToString());
        }

        private void AddMessage(string message)
        {
            TextBlock tb = new TextBlock();
            tb.Text = message;
            _statusMessages.Children.Add(tb);
        }
    }
}
